#
# $Id: mssql.pm,v 1.1 2003-12-17 17:21:46 rodney Exp $
#
#
#



# ------------------------------------------------------------------
package RvSubDbi::mysql;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi);





# ------------------------------------------------------------------
package RvSubDbi::mysql::st;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi::st);





# ------------------------------------------------------------------
package RvSubDbi::mysql::db;
use strict;
use vars qw(@ISA);
use DBI qw(:sql_types);
@ISA = qw(RvSubDbi::db);


#
# Die Typemap bildet ein Mapping von DB-spezifischen Types auf
# DBI-Types.
#
# Format: [ { code => SQL_SOWIESO, # DBI-Type gem. sql_types
#             name => 'NUMBER',    # Plattform-spezifischer Type-Name
#           } ]
#
# Von mehreren Datentypen mit gleichem 'code' wird der letzte
# verwendet, wenn von 'code' auf 'name' gemapped wird.
#
my $typemap = [ 
		{ code => SQL_NUMERIC, name => 'smallint' },
		{ code => SQL_NUMERIC, name => 'bigint' },
		{ code => SQL_NUMERIC, name => 'decimal' },
		{ code => SQL_VARCHAR, name => 'blob' },
		{ code => SQL_VARCHAR, name => 'text' },
		{ code => SQL_VARCHAR, name => 'varchar' },
		{ code => SQL_FLOAT, name => 'float' },
		];


sub datatypeByPlatformTypename
{
    my ($self,$c) = @_;

    my %map = map { $_->{name} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype $c") unless ($r);

    return($r);
}



# Parameter:
#
#      {
#	COLUMN_NAME => "description",
#	DATA_TYPE => $dtype->{code}, # DBI-Typename
#	NOTNULL => 1 | 0,
#	SCALE => $scale,
#	PRECISION => $prec
#      }
#
sub createLine
{
    my ($self, $h) = @_;


    # MySQL needs to convert VARCHAR fields to TEXT datatype.
    if ($h->{DATA_TYPE} == SQL_VARCHAR)
    {
	my $cmd = $h->{COLUMN_NAME} . " text";

	$cmd .= " NOT NULL" if ($h->{NOTNULL});
	
	return($cmd);
    }


    return($self->SUPER::createLine($h));
}




sub selectLine
{
    my ($self, $h) = @_;




    return($self->SUPER::selectLine($h));
}


sub insertLine
{
    my ($self,$h) = @_;

    if (! $h->{DATA_TYPE})
    {
	die("missing data type for $h->{COLUMN_NAME}");
    }


    return($self->SUPER::insertLine($h));
}


#
# Return the typemap entry for a given SQL_something Type
#
sub PlatformType
{
    my ($self,$c) = @_;

    my %map = map { $_->{code} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype $c") unless ($r);

    return($r);
}




sub table_attributes
{
    my ($self, $table) = @_;


    my $cmd = "SELECT * FROM $table LIMIT 5";


    print "mysql doing: $cmd\n"; # DEBUG

    my $sth = $self->prepare($cmd) or die $self->errstr;
    $sth->execute or die $sth->errstr;
    my @ret;

    my $names = $sth->{'NAME'};
    my $types = $sth->{'TYPE'};
    my $size = $sth->{'PRECISION'};
    my $scale = $sth->{'SCALE'};
    my $pri = $sth->{'mysql_is_pri_key'};
    my $mysql_length = $sth->{'mysql_length'}; # max. currently present value
    my $nullable = $sth->{'NULLABLE'};
    my $tnames = $sth->{'mysql_type_name'};
    my $numFields = $sth->{'NUM_OF_FIELDS'};

    for (my $i = 0;  $i < $numFields;  $i++)
    {
	printf("%s t=%s tn=%s\n", $$names[$i], $$types[$i], $$tnames[$i]);

	my $dtype = $self->datatypeByPlatformTypename($$tnames[$i]);

	my $prec = $$size[$i];
	my $scale = $$scale[$i];

#	print "prec $prec, scale $scale\n";

	# FIXME: MySQL seems to report size + 1 for numeric types
	if ($dtype->{code} == SQL_NUMERIC)
	{
#	    $prec -= 1;
	}
	elsif ($dtype->{code} == SQL_FLOAT)
	{
	    $scale = undef; # Float does not have a scale, per definition
	}
#	elsif ($dtype->{code} == SQL_VARCHAR)
#	{
#	    $prec = undef;
#	}

	# Rueckgabe in dem Format, das in DBI.1 unter
	#    "Statement Handle Attributes"
	# dokumentiert ist.
	my $r = {
	    TABLE_CAT => undef,
	    TABLE_SCHEM => undef,
	    TABLE_NAME => $table,
	    COLUMN_NAME => $$names[$i],
	    DATA_TYPE => $dtype->{code}, # DBI-Typename
	    TYPE_NAME => $dtype->{name}, # Platform-specific name
	    PRIMARY_KEY => ($$pri[$i] ? 1 : 0),
	    NOTNULL => ($$nullable[$i] ? 0 : 1),
	    SCALE => $scale,
	    PRECISION => $prec,
	};


	push(@ret,$r);

	
    }

    return(\@ret);

}











sub Xtable_attributes
{
    my ($self, $table) = @_;

    my $schema = "USER";# unless ($schema);

    my $cmd = "SELECT column_name, data_type, data_length, nullable,
                      data_default, data_precision, data_scale, USER schema
                 FROM user_tab_columns
	        WHERE upper(table_name) = upper('$table')
	        ORDER BY COLUMN_ID";


    print "doing: $cmd\n";

    my $sth = $self->prepare($cmd) or die $self->errstr;
    $sth->execute or die $sth->errstr;
    my $data = $sth->fetchall_arrayref();


    my @ret;

    foreach my $d (@$data)
    {
	my $dtype = $self->datatypeByPlatformTypename($d->[1]);

	# Rueckgabe in dem Format, das in DBI.1 unter
	#    "Statement Handle Attributes"
	# dokumentiert ist.
	my $r = {
	    TABLE_CAT => undef,
	    TABLE_SCHEM => $d->[7], # Schema name
	    TABLE_NAME => $table,
	    COLUMN_NAME => $d->[0],
	    SIZE => $d->[2],
	    DATA_TYPE => $dtype->{code}, # DBI-Typename
	    TYPE_NAME => $dtype->{name},
	    PRIMARY_KEY => 0,
	    NOTNULL => ($d->[3] eq 'Y' ? 0 : 1),
	    SCALE => $d->[6],
	    PRECISION => $d->[5],

	};

	# Datatype-specific SCALE/PRECISION setting
	if ($dtype->{code} == SQL_VARCHAR)
	{
	    $r->{PRECISION} = $r->{SIZE};
	}




	push(@ret,$r);

    }

    return(\@ret);

}



