#
# $Id: odbc_mssql.pm,v 1.1 2003-11-24 16:33:20 rodney Exp $
#



# ------------------------------------------------------------------
package RvSubDbi::odbc_mssql;
use strict;
use vars qw(@ISA);
@ISA = qw(DBI);

# Tell DBI that our package is a new root class:
# RvSubDbi::odbc_mssql->init_rootclass(); # taken from subclass.t.
#__PACKAGE__->init_rootclass; # More general approach.


# ------------------------------------------------------------------
package RvSubDbi::odbc_mssql::st;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi::st);





# ------------------------------------------------------------------
package RvSubDbi::odbc_mssql::db;
use strict;
use vars qw(@ISA);
use DBI qw(:sql_types);
@ISA = qw(RvSubDbi::db);

#@DBI::odbc_mssql::db::ISA = qw(DBI::db); # From ProxyServer.pm


#
# Die Typemap bildet ein Mapping von DB-spezifischen Types auf
# DBI-Types.
#
# Format: [ { code => SQL_SOWIESO, # DBI-Type gem. sql_types
#             name => 'NUMBER',    # Plattform-spezifischer Type-Name
#           } ]
#
# Von mehreren Datentypen mit gleichem 'code' wird der letzte
# verwendet, wenn von 'code' auf 'name' gemapped wird.
#
my $typemap = [ 
		{ code => SQL_LONGVARCHAR,
		  name => 'text',
		  mscode => -1 },
		{ code => SQL_CHAR,
		  name => 'char',
		  mscode => 1 },
		{ code => SQL_DATETIME,
		  name => 'datetime',
		  mscode => 11 },
		{ code => SQL_NUMERIC,
		  name => 'number',
		  mscode => -7 },
		{ code => SQL_NUMERIC,
		  name => 'decimal',
		  mscode => 3 },
		{ code => SQL_NUMERIC,
		  name => 'decimal',
		  mscode => 4 },
		{ code => SQL_VARCHAR,
		  name => 'varchar',
		  mscode => 12 },
		];


sub datatypeByPlatformTypename
{
    my ($self,$c) = @_;

    my %map = map { $_->{name} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype '$c'") unless ($r);

    return($r);
}



# Parameter:
#
#      {
#	COLUMN_NAME => "description",
#	DATA_TYPE => $dtype->{code}, # DBI-Typename
#	NOTNULL => 1 | 0,
#	SCALE => $scale,
#	PRECISION => $prec
#      }
#
sub createLine
{
    my ($self, $h) = @_;


    if ($h->{DATA_TYPE} == SQL_LONGVARCHAR)
    {
	my $cmd = $h->{COLUMN_NAME} . " text";

	$cmd .= " NOT NULL" if ($h->{NOTNULL});
	
	return($cmd);
    }
    elsif ($h->{DATA_TYPE} == SQL_DATETIME)
    {
	my $cmd = $h->{COLUMN_NAME} . " datetime";

	$cmd .= " NOT NULL" if ($h->{NOTNULL});
	
	return($cmd);
    }


    return($self->SUPER::createLine($h));
}




sub selectLine
{
    my ($self, $h) = @_;

    return($self->SUPER::selectLine($h));
}


sub insertLine
{
    my ($self,$h) = @_;

    if (! $h->{DATA_TYPE})
    {
	die("missing data type for $h->{COLUMN_NAME}");
    }


    return($self->SUPER::insertLine($h));
}


#
# Return the typemap entry for a given SQL_something Type
#
sub PlatformType
{
    my ($self,$c) = @_;

    my %map = map { $_->{code} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype $c") unless ($r);

    return($r);
}




sub table_attributes
{
    my ($self, $table) = @_;

#    print "odbc_mssql->table_attributes($table)\n"; # DEBUG


    if (1) # DEBUG
    {
	# Show a list of tables in the MSSQL Database
	my @tabs = $self->tables();
	
	foreach my $t (@tabs)
	{
	    print "$t\n";
	}
	exit(0);
    }


    
#    my $s = $self->columns;

#    print "got $s\n";


    # Generate dummy sth for this table
    my $cmd = "SELECT * FROM $table WHERE 1 = 0";
    my $sth = $self->prepare($cmd) or die $self->errstr;
    $sth->execute or die $sth->errstr;


    if (0) # DEBUG
    {
	print "START sth dump -------------------------\n";
	foreach my $k (keys %$sth)
	{
	    print "$k '$sth->{$k}'\n";
	}
	print "END sth dump -------------------------\n";
#	exit(0);
    }

    
    my $names = $sth->{'NAME'};
    my $types = $sth->{'TYPE'};
    my $size = $sth->{'PRECISION'};
    my $scale = $sth->{'SCALE'};
    my $pri = $sth->{'mysql_is_pri_key'};
    my $mysql_length = $sth->{'mysql_length'}; # max. currently present value
    my $nullable = $sth->{'NULLABLE'};

#    my $tnames = $sth->{'mysql_type_name'};


    my %tnames_h = map { $_->{mscode} => $_ } @$typemap;
    my $tnames = \%tnames_h;
    
    

    my $numFields = $sth->{'NUM_OF_FIELDS'};


#    print "names: $names\n";

    if (0)
    {
	foreach my $k (@$names)
	{
	    print "$k\n";
	}
    }



    my @ret;


    for (my $i = 0;  $i < $numFields;  $i++)
    {

	my $typename = $tnames->{$$types[$i]}->{name};
	printf("%s t=%s tn=%s\n", $$names[$i],
	       $$types[$i],
	       $typename);

	my $dtype = $self->datatypeByPlatformTypename($typename);

	my $prec = $$size[$i];
	my $scale = $$scale[$i];

#	print "prec $prec, scale $scale\n";

	# FIXME: MySQL seems to report size + 1 for numeric types
	if ($dtype->{code} == SQL_NUMERIC)
	{
#	    $prec -= 1;
	}
	elsif ($dtype->{code} == SQL_FLOAT)
	{
	    $scale = undef; # Float does not have a scale, per definition
	}
#	elsif ($dtype->{code} == SQL_VARCHAR)
#	{
#	    $prec = undef;
#	}

	# Rueckgabe in dem Format, das in DBI.1 unter
	#    "Statement Handle Attributes"
	# dokumentiert ist.
	my $r = {
	    TABLE_CAT => undef,
	    TABLE_SCHEM => undef,
	    TABLE_NAME => $table,
	    COLUMN_NAME => $$names[$i],
	    DATA_TYPE => $dtype->{code}, # DBI-Typename
	    TYPE_NAME => $dtype->{name}, # Platform-specific name
	    PRIMARY_KEY => ($$pri[$i] ? 1 : 0),
	    NOTNULL => ($$nullable[$i] ? 0 : 1),
	    SCALE => $scale,
	    PRECISION => $prec,
	};


	push(@ret,$r);
    }

    return(\@ret);
}











sub Xtable_attributes
{
    my ($self, $table) = @_;

    my $schema = "USER";# unless ($schema);

    my $cmd = "SELECT column_name, data_type, data_length, nullable,
                      data_default, data_precision, data_scale, USER schema
                 FROM user_tab_columns
	        WHERE upper(table_name) = upper('$table')
	        ORDER BY COLUMN_ID";


    print "doing: $cmd\n";

    my $sth = $self->prepare($cmd) or die $self->errstr;
    $sth->execute or die $sth->errstr;
    my $data = $sth->fetchall_arrayref();


    my @ret;

    foreach my $d (@$data)
    {
	my $dtype = $self->datatypeByPlatformTypename($d->[1]);

	# Rueckgabe in dem Format, das in DBI.1 unter
	#    "Statement Handle Attributes"
	# dokumentiert ist.
	my $r = {
	    TABLE_CAT => undef,
	    TABLE_SCHEM => $d->[7], # Schema name
	    TABLE_NAME => $table,
	    COLUMN_NAME => $d->[0],
	    SIZE => $d->[2],
	    DATA_TYPE => $dtype->{code}, # DBI-Typename
	    TYPE_NAME => $dtype->{name},
	    PRIMARY_KEY => 0,
	    NOTNULL => ($d->[3] eq 'Y' ? 0 : 1),
	    SCALE => $d->[6],
	    PRECISION => $d->[5],

	};

	# Datatype-specific SCALE/PRECISION setting
	if ($dtype->{code} == SQL_VARCHAR)
	{
	    $r->{PRECISION} = $r->{SIZE};
	}




	push(@ret,$r);

    }

    return(\@ret);

}



=cut


- Beispiele fuer dbish-Zugriff auf ODBC-MSSQL Datasource:

  CREATE TABLE bla (c CHAR(3)) /
  INSERT INTO bla (c) VALUES ('a') /
  INSERT INTO bla (c) VALUES ('b') /
  INSERT INTO bla (c) VALUES ('c') /
  COMMIT /
  SELECT * FROM bla /


- Zugriff auf Tables anderer User:

  SELECT * from dbo.firma /
  SELECT * from dbo.katalog /





