#
# $Id: oracle.pm,v 1.6 2004-05-26 10:20:39 rodney Exp $
#
#
#



# ------------------------------------------------------------------
package RvSubDbi::oracle;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi);





# ------------------------------------------------------------------
package RvSubDbi::oracle::st;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi::st);





# ------------------------------------------------------------------
package RvSubDbi::oracle::db;
use strict;
use vars qw(@ISA);
use DBI qw(:sql_types);
@ISA = qw(RvSubDbi::db);


#
# Die Typemap bildet ein Mapping von DB-spezifischen Types auf
# DBI-Types.
#
# Format: [ { code => SQL_SOWIESO, # DBI-Type gem. sql_types
#             name => 'NUMBER',    # Plattform-spezifischer Type-Name
#           } ]
#
my $typemap = [ 
		{ code => SQL_NUMERIC, name => 'NUMBER' },
		{ code => SQL_VARCHAR, name => 'VARCHAR' },
		{ code => SQL_VARCHAR, name => 'VARCHAR2' },
		{ code => SQL_DATETIME, name => 'DATE' },
		{ code => SQL_CLOB,    name => 'CLOB' },
		{ code => SQL_FLOAT,   name => 'FLOAT' },
		];


sub fieldQuote
{
    my ($self,$n) = @_;

    return('"' . $n . '"'); # Oracle needs quoted fieldnames
}




sub datatypeByPlatformTypename
{
    my ($self,$c) = @_;

    my %map = map { $_->{name} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype $c") unless ($r);

    return($r);
}



#
# Return the typemap entry for a given SQL_something Type
#
sub PlatformType
{
    my ($self,$c) = @_;

    my %map = map { $_->{code} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype $c") unless ($r);

    return($r);
}




sub selectLine
{
    my ($self,$h) = @_;

    if (! $h->{DATA_TYPE})
    {
	die("missing data type for $h->{COLUMN_NAME}");
    }

    # Fuer SQL_DATETIME brauchen wir eine Conversion
    if ($h->{DATA_TYPE} == SQL_DATETIME)
    {
	return("o2unixdate($h->{COLUMN_NAME}) $h->{COLUMN_NAME}");
    }

    return($self->SUPER::selectLine($h));
}


sub insertLine
{
    my ($self,$h) = @_;

    if (! $h->{DATA_TYPE})
    {
	die("missing data type for $h->{COLUMN_NAME}");
    }

    # Fuer SQL_DATETIME brauchen wir eine Conversion
    if ($h->{DATA_TYPE} == SQL_DATETIME)
    {
	return("unix2odate(?)");
    }

    return($self->SUPER::insertLine($h));
}


sub table_attributes
{
    my ($self, $table) = @_;

    my $schema = "USER";# unless ($schema);

    my $cmd = "SELECT column_name, data_type, data_length, nullable,
                      data_default, data_precision, data_scale, USER schema
                 FROM user_tab_columns
	        WHERE upper(table_name) = upper('$table')
	        ORDER BY COLUMN_ID";


    print "doing: $cmd\n";

    my $sth = $self->prepare($cmd) or die $self->errstr;
    $sth->execute or die $sth->errstr;
    my $data = $sth->fetchall_arrayref();


    my @ret;

    foreach my $d (@$data)
    {
	my $dtype = $self->datatypeByPlatformTypename($d->[1]);

	# Rueckgabe in dem Format, das in DBI.1 unter
	#    "Statement Handle Attributes"
	# dokumentiert ist.
	my $r = {
	    TABLE_CAT => undef,
	    TABLE_SCHEM => $d->[7], # Schema name
	    TABLE_NAME => $table,
	    COLUMN_NAME => $d->[0],
	    SIZE => $d->[2],
	    DATA_TYPE => $dtype->{code}, # DBI-Typename
	    TYPE_NAME => $dtype->{name},
	    PRIMARY_KEY => 0,
	    NOTNULL => ($d->[3] eq 'Y' ? 0 : 1),
	    SCALE => $d->[6],
	    PRECISION => $d->[5],

	};

	# Datatype-specific SCALE/PRECISION setting
	if ($dtype->{code} == SQL_VARCHAR)
	{
	    $r->{PRECISION} = $r->{SIZE};
	}




	push(@ret,$r);

    }

    return(\@ret);

}

