#
# $Id: postgresql.pm,v 1.5 2003-12-17 17:21:25 rodney Exp $
#
#
#



# ------------------------------------------------------------------
package RvSubDbi::postgresql;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi);





# ------------------------------------------------------------------
package RvSubDbi::postgresql::st;
use strict;
use vars qw(@ISA);
@ISA = qw(RvSubDbi::st);





# ------------------------------------------------------------------
package RvSubDbi::postgresql::db;
use strict;
use vars qw(@ISA);
use DBI qw(:sql_types);
@ISA = qw(RvSubDbi::db);


#
# Die Typemap bildet ein Mapping von DB-spezifischen Types auf
# DBI-Types.
#
# Format: [ { code => SQL_SOWIESO, # DBI-Type gem. sql_types
#             name => 'NUMBER',    # Plattform-spezifischer Type-Name
#           } ]
#
my $typemap = [ 
		{ code => SQL_NUMERIC, name => 'numeric' },
		{ code => SQL_VARCHAR, name => 'char' },
		{ code => SQL_VARCHAR, name => 'varchar' },
		{ code => SQL_VARCHAR, name => 'char' },
		{ code => SQL_DATETIME, name => 'timestamp' },
		{ code => SQL_FLOAT, name => 'float' },
		];


sub datatypeByPlatformTypename
{
    my ($self,$c) = @_;

    my %map = map { $_->{name} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype name '$c'") unless ($r);

    return($r);
}



#
# Return the typemap entry for a given SQL_something Type
#
sub PlatformType
{
    my ($self,$c) = @_;

    my %map = map { $_->{code} => $_ } @$typemap;

    my $r = $map{$c};

    die("unknown datatype $c") unless ($r);

    return($r);
}




sub selectLine
{
    my ($self,$h) = @_;

    if (! $h->{DATA_TYPE})
    {
	die("missing data type for $h->{COLUMN_NAME}");
    }

    # Fuer SQL_DATETIME brauchen wir eine Conversion
    if ($h->{DATA_TYPE} == SQL_DATETIME)
    {
	return("date_part('epoch',$h->{COLUMN_NAME}) AS $h->{COLUMN_NAME}");
    }

    return($self->SUPER::selectLine($h));
}


sub insertLine
{
    my ($self,$h) = @_;

    if (! $h->{DATA_TYPE})
    {
	die("missing data type for $h->{COLUMN_NAME}");
    }

    # Fuer SQL_DATETIME brauchen wir eine Conversion
    if ($h->{DATA_TYPE} == SQL_DATETIME)
    {
	return("('epoch'::timestamp + ?::interval)");
    }

    return($self->SUPER::insertLine($h));
}


sub table_attributes
{
    my ($self, $table) = @_;


    my $data = $self->func($table, 'table_attributes');

    my @ret;

    foreach my $d (@$data)
    {

	if (1) # DEBUG
	{
	    print "table_attributes for table '$table' -------------------\n";
	    foreach my $k (%$d)
	    {
		print "'$k' => '$d->{$k}'\n";
	    }
	}

	
	# Precision/Scale
	my $scale = undef;
	my $prec = ($d->{SIZE} > 0 ? $d->{SIZE} : undef);

	my $dtype = $self->datatypeByPlatformTypename($d->{TYPE});
#
#	# Rueckgabe in dem Format, das in DBI.1 unter
#	#    "Statement Handle Attributes"
#	# dokumentiert ist.
	my $r = {
	    TABLE_CAT => undef,
	    TABLE_SCHEM => undef, # Schema name
	    TABLE_NAME => undef,
	    COLUMN_NAME => $d->{NAME},
	    SIZE => undef,
	    DATA_TYPE => $dtype->{code}, # DBI-Typecode
	    TYPE_NAME => $dtype->{name}, # Platform-specific typename
	    PRIMARY_KEY => undef,
	    NOTNULL => $d->{NOTNULL},
	    SCALE => $scale,
	    PRECISION => $prec,
	};
#
	# Datatype-specific SCALE/PRECISION setting
	if ($dtype->{code} == SQL_DATETIME)
	{
	    $r->{PRECISION} = undef;;
	}

	push(@ret,$r);
    }

    return(\@ret);
}

